/*
 * main.c
 *
 *  Created on: 2013-03-23
 *       Autor: ukasz Dachowski
 *       Autorem bibliotek jest Mirosaw Karda
 *       Biblioteki znajduj si na pycie doczonej do ksiki "Mikrokontrolery AVR Jzyk C Podstawy programowania"
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "LCD/lcd44780.h"
#include "IR_DECODE/ir_decode.h"
#include "1Wire/ds18x20.h"

//******************************* definicje preprocesora


#define LED1 (1<<PC6)	// zwyka dioda LED
#define LED2 (1<<PA7)	// podwietlenie LCD

#define PM1 (1<<PC0)	// pwm
#define PM2 (1<<PC1)

#define PM3 (1<<PC2)	//zalicza do PZ
#define PM4 (1<<PC3)
#define PZ3 (1<<PC4)
#define PZ4 (1<<PC5)

#define LED1_OFF PORTC &= ~LED1
#define LED1_ON PORTC |= LED1
#define LED1_TOG PORTC ^= LED1
#define LED2_ON PORTA |= LED2
#define LED2_OFF PORTA &= ~LED2
#define LED2_TOG PORTA ^= LED2

#define LED1_DDR DDRC
#define LED2_DDR DDRA


#define PM_DDR DDRC

#define PM1_ON PORTC |= PM1   //pwm
#define PM1_OFF PORTC &= ~PM1

#define PM2_ON PORTC |= PM2
#define PM2_OFF PORTC &= ~PM2
#define PM3_ON PORTC |= PM3    //(to i dalej) zalicza do PZ
#define PM3_OFF PORTC &= ~PM3
#define PM4_ON PORTC |= PM4
#define PM4_OFF PORTC &= ~PM4
#define PZ3_ON PORTC |= PZ3
#define PZ3_OFF PORTC &= ~PZ3
#define PZ4_ON PORTC |= PZ4
#define PZ4_OFF PORTC &= ~PZ4


#define KL1 (1<<PD3)
#define KL2 (1<<PD4)
#define KL3 (1<<PD5)
#define KL4 (1<<PD7)
#define KL_DDR DDRD
#define KL_PORT PORTD

volatile uint8_t pwm1, pwm2;

//******************************* deklaracje funkcji
void display_temp(uint8_t x);
// konwersja liczby dziesitnej na BCD
uint8_t dec2bcd(uint8_t dec);
// konwersja liczby BCD na dziesitn
uint8_t bcd2dec(uint8_t bcd);

//******************************* deklaracje funkcji


/************** funkcja SuperDebounce do obsugi pojedynczych klawiszy ***************/
void SuperDebounce(uint8_t * key_state, volatile uint8_t *KPIN,
		uint8_t key_mask, uint16_t rep_time, uint16_t rep_wait,
		void (*push_proc)(void), void (*rep_proc)(void) );
void kl1_press(void);
void kl1_rep(void);
void kl2_press(void);
void kl2_rep(void);

//******************************* definicje zmiennych globalnych i typw
uint8_t subzero, cel, cel_fract_bits;
uint8_t czujniki_cnt;		/* ilo czujnikw na magistrali */

uint8_t stopien[] = {12,18,18,12,32,32,32,32};
volatile uint16_t Timer1, Timer2, Timer3, Timer4;	/* timery programowe 100Hz */

uint8_t lcd_blink;	/* zmienna odpowiedzialna za zaczanie migotania podwietlenia LCD */
uint8_t lcd_cnt;	/* zmienna licznik do wywietlania na LCD */
uint8_t sek;

/*----------------------------- gwna funkcja programu  ---------------*/
int main(void) {


/*----------------------------- inicjalizacja kierunkw pinw  ---------------*/
	// ustalamy kierunki portw klawiszy na wyjciowe
	KL_DDR 	&= ~(KL1|KL2|KL3|KL4);
	// wczamy podciganie do VCC
	KL_PORT |= KL1|KL2|KL3|KL4;

	// ustalamy kierunki pinw diod LED
	LED1_DDR |= LED1; LED2_DDR |= LED2; PM_DDR |= PM1;
	PM_DDR |= PM2; PM_DDR |= PM3; PM_DDR |= PM4; PM_DDR |= PZ3; PM_DDR |= PZ4;
	// wyczay diod
	LED1_OFF;
	// zaczamy podwietlenie
	LED2_ON;



	/* Timer2  inicjalizacja przerwania co 10ms */
	TCCR2 	|= (1<<WGM21);			// tryb pracy CTC
	TCCR2 	|= (1<<CS22)|(1<<CS21)|(1<<CS20);	// preskaler = 1024
	OCR2 	= 156;					// przerwanie porwnania co 10ms (100Hz)
	TIMSK 	= (1<<OCIE2);			// Odblokowanie przerwania CompareMatch

	// ustawienia TIMER0 w tryb CTC
	TCCR0 |= (1<<WGM01);	// tryb  CTC
	TCCR0 |= (1<<CS01);		// preskaler = 8
	OCR0 = 80;				// dodatkowy podzia czsttotliwoci przez 40
	TIMSK |= (1<<OCIE0);	// zezwolenie na przerwanie CompareMatch


/*----------------------------- inicjalizacja moduw programowych  ---------------*/

	/* inicjalizacja LCD */
	lcd_init();
	lcd_str("Start...");
	lcd_defchar(0x80,stopien); // definicja znaku stopnia w LCD


	/* inicjalizacja odbiornika podczerwieni RC5 */
	ir_init();



	/* globalne  odblokowanie przerwa */
	sei();

	/* sprawdzamy ile czujnikw DS18xxx widocznych jest na magistrali */
	czujniki_cnt = search_sensors();
	DS18X20_start_meas( DS18X20_POWER_PARASITE, NULL );

	/* czekamy 750ms na dokonanie konwersji przez podczone czujniki */
	_delay_ms(750);

	lcd_cls();

	/* dokonujemy odczytu temperatury z pierwszego czujnika o ile zosta wykryty */
	/* wywietlamy temperatur gdy czujnik wykryty */
	if( DS18X20_OK == DS18X20_read_meas(gSensorIDs[0], &subzero, &cel, &cel_fract_bits) ) display_temp(0);

	uint8_t k1, k2;

	while(1) {


		SuperDebounce(&k1, &PIND, KL1, 20, 500, kl1_press, kl1_rep );
		SuperDebounce(&k2, &PIND, KL2, 80, 400, kl2_press, kl2_rep );

		if(!Timer2) {
				Timer2=25;
				if( lcd_blink ) LED2_TOG;
		}



		/* ****** ZDARZENIE Z ODBIORNIKA PODCZERWIENI ********** */
		if(Ir_key_press_flag) {



			if( !address && 1==command )
			{
				pwm1=70;
				pwm2=70;
				PM3_ON;
				PM4_OFF;
				PZ3_ON;
				PZ4_OFF;
			}
			if( !address && 2==command )
			{
				pwm1=100;
				pwm2=100;
				PM3_ON;
				PM4_OFF;
				PZ3_ON;
				PZ4_OFF;
			}
			if( !address && 3==command )
			{
				pwm1=170;
				pwm2=170;
				PM3_ON;
				PM4_OFF;
				PZ3_ON;
				PZ4_OFF;
			}
			if( !address && 4==command )
			{
				pwm1=200;
				pwm2=200;
				PM3_ON;
				PM4_OFF;
				PZ3_ON;
				PZ4_OFF;
			}

			if( !address && 32==command ) //kierunki(CH i vol) CH+
			{
				pwm1=255;
				PM3_ON;
				PM4_OFF;
			}
			if( !address && 33==command ) //kierunki(CH i vol) CH-
			{
				pwm1=255;
				PM3_OFF;
				PM4_ON;
			}
			if( !address && 16==command ) //kierunki(CH i vol) vol+
			{
				pwm2=255;
				PZ3_ON;
				PZ4_OFF;
			}
			if( !address && 17==command ) //kierunki(CH i vol) vol-
			{
				pwm2=255;
				PZ3_OFF;
				PZ4_ON;
			}


			if( !address && 5==command ) pwm1=0, pwm2=0, PM3_OFF, PM4_OFF, PZ3_OFF, PZ4_OFF;

			if( !address && 6==command )
			{
				pwm1=255;
				pwm2=255;
				PM3_ON;
				PM4_OFF;
				PZ3_ON;
				PZ4_OFF;
			}
			if( !address && 8==command )
			{
				pwm1=115;
				pwm2=115;
				PM3_OFF;
				PM4_ON;
				PZ3_OFF;
				PZ4_ON;
			}
			if( !address && 7==command )
			{
				pwm1=200;
				pwm2=70;
				PM3_ON;
				PM4_OFF;
				PZ3_ON;
				PZ4_OFF;
			}
			if( !address && 9==command )
			{
				pwm1=70;
				pwm2=200;
				PM3_ON;
				PM4_OFF;
				PZ3_ON;
				PZ4_OFF;
			}

			if( !address && 10==command )
			{
				pwm1=140;
				pwm2=140;
				PM3_OFF;
				PM4_ON;
				PZ3_OFF;
				PZ4_ON;
			}
			if( !address && 0==command )
			{
				pwm1=255;
				pwm2=255;
				PM3_OFF;
				PM4_ON;
				PZ3_OFF;
				PZ4_ON;
			}



			lcd_locate(0,0);
			lcd_str("L ");
			lcd_int(pwm1);
			lcd_str("  ");
			lcd_locate(0,6);
			lcd_str("P ");
			lcd_int(pwm2);
			lcd_str("  ");

			lcd_locate(1,8);
			lcd_str_P(PSTR("rc5: "));
			lcd_int(command);
			lcd_str_P(PSTR("  "));

			if( !address && 50==command ) lcd_blink = 1;	// zaczamy miganie podwietlenia LCD
			if( !address && 11==command ) lcd_blink = 0;	// wyczamy miganie podwietlenia LCD
			if( !address && 52==command ) LED2_ON; // wczamy podwietlenie LCD
			if( !address && 55==command ) LED2_OFF; // wyczamy podwietlenie LCD

			Ir_key_press_flag = 0;
			command=0xff;
			address=0xff;
		}
		if (!Timer3) {
			Timer3=100;


			if(!sek) czujniki_cnt = search_sensors();
			if(1==sek) DS18X20_start_meas( DS18X20_POWER_PARASITE, NULL );

			if(2==sek) {
				if( DS18X20_OK == DS18X20_read_meas(gSensorIDs[0], &subzero, &cel, &cel_fract_bits) ) display_temp(0);
				else {
					lcd_locate(1,0);
					lcd_str(" error ");
				}
			}
			if(++sek>2) sek=0;
		}


	}
}


 ISR(TIMER0_COMP_vect)
{
	static uint8_t cn; // definicja naszego licznika PWM

	// bezporednie sterowanie wyjciami kanaw PWM
	if(cn<=pwm1) PM1_ON; else PM1_OFF;
	if(cn<=pwm2) PM2_ON; else PM2_OFF;

	cn++;	// zwikszanie licznika o 1
}



ISR(TIMER2_COMP_vect)
{
	uint16_t n;

	n = Timer1;		/* 100Hz Timer1 */
	if (n) Timer1 = --n;
	n = Timer2;		/* 100Hz Timer2 */
	if (n) Timer2 = --n;
	n = Timer3;		/* 100Hz Timer3 */
	if (n) Timer3 = --n;
	n = Timer4;		/* 100Hz Timer4 */
	if (n) Timer4 = --n;
}



/************** funkcja SuperDebounce do obsugi pojedynczych klawiszy ***************
 * 							AUTOR: Mirosaw Karda
 * ZALETY:
 * 		- nie wprowadza najmniejszego spowalnienia
 * 		- posiada funkcj REPEAT (powtarzanie akcji dla duej wcinitego przycisku)
 * 		- mona przydzieli rne akcje dla trybu REPEAT i pojedynczego kliknicia
 * 		- mona przydzieli tylko jedn akcj wtedy w miejsce drugiej przekazujemy 0 (NULL)
 * 		- zabezpieczenie przed naciniciem dwch klawiszy jednoczenie
 *
 * Wymagania:
 * 	Timer programowy utworzony w oparciu o Timer sprztowy (przerwanie 100Hz)
 *
 * 	Parametry wejciowe - 6 :
 *  *KPIN - nazwa PINx portu na ktrym umieszczony jest klawisz, np: PINB
 *  key_mask - maska klawisza np: (1<<PB3)
 *  rep_time - czas powtarzania funkcji rep_proc w trybie REPEAT
 *  rep_wait - czas oczekiwania do przejcia do trybu REPEAT
 *  push_proc - wskanik do wasnej funkcji wywoywanej raz po zwolenieniu przycisku
 *  rep_proc - wskanik do wasnej funkcji wykonywanej w trybie REPEAT
 **************************************************************************************/
void SuperDebounce(uint8_t * key_state, volatile uint8_t *KPIN,
		uint8_t key_mask, uint16_t rep_time, uint16_t rep_wait,
		void (*push_proc)(void), void (*rep_proc)(void) ) {

	enum {idle, debounce, go_rep, wait_rep, rep};

	if(!rep_time) rep_time=20;
	if(!rep_wait) rep_wait=150;

	uint8_t key_press = !(*KPIN & key_mask);

	if( key_press && !*key_state ) {
		*key_state = debounce;
		Timer1 = 15;
	} else
	if( *key_state  ) {

		if( key_press && debounce==*key_state && !Timer1 ) {
			*key_state = 2;
			Timer1=5;
		} else
		if( !key_press && *key_state>1 && *key_state<4 ) {
			if(push_proc) push_proc();						/* KEY_UP */
			*key_state=idle;
		} else
		if( key_press && go_rep==*key_state && !Timer1 ) {
			*key_state = wait_rep;
			Timer1=rep_wait;
		} else
		if( key_press && wait_rep==*key_state && !Timer1 ) {
			*key_state = rep;
		} else
		if( key_press && rep==*key_state && !Timer1 ) {
			Timer1 = rep_time;
			if(rep_proc) rep_proc();						/* KEY_REP */
		}
	}
	if( *key_state>=3 && !key_press ) *key_state = idle;
}



void kl1_press(void) {
	LED1_TOG;
}

void kl1_rep(void) {
	LED1_OFF;
}

void kl2_press(void) {
	LED1_ON;
}

void kl2_rep(void) {
	lcd_locate(0,0);
	lcd_str_P(PSTR("cnt "));
	lcd_int(lcd_cnt++);
	lcd_str_P(PSTR("  "));
}

/* wywietlanie temperatury na pozycji X w drugiej linii LCD */
void display_temp(uint8_t x) {
	lcd_locate(1,x);
	if(subzero) lcd_str("-");	/* jeli subzero==1 wywietla znak minus (temp. ujemna) */
	else lcd_str(" ");	/* jeli subzero==0 wywietl spacj zamiast znaku minus (temp. dodatnia) */
	lcd_int(cel);	/* wywietl dziesitne czci temperatury  */
	lcd_str(".");	/* wywietl kropk */
	lcd_int(cel_fract_bits); /* wywietl dziesitne czci stopnia */
	lcd_str("\x80""C "); /* wywietl znak jednostek (C - stopnie Celsiusza) */
}

